#!/usr/bin/python
#-*- coding: utf-8 -*-

"""
SAT communication bridge
Copyright (C) 2009, 2010, 2011, 2012  Jérôme Poisson (goffi@goffi.org)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from bridge_frontend import BridgeFrontend
import dbus, dbus.glib
from logging import debug, error

const_INT_PREFIX = "org.goffi.SAT"  #Interface prefix
const_ERROR_PREFIX = const_INT_PREFIX+".error"
const_OBJ_PATH = '/org/goffi/SAT/bridge'
const_CORE_SUFFIX = ".core"
const_PLUGIN_SUFFIX = ".plugin"

class BridgeExceptionNoService(Exception):
    pass

class DBusBridgeFrontend(BridgeFrontend):
    def __init__(self):
        try:
            self.sessions_bus = dbus.SessionBus()
            self.db_object = self.sessions_bus.get_object(const_INT_PREFIX,
                                const_OBJ_PATH)
            self.db_core_iface = dbus.Interface(self.db_object,
                                dbus_interface=const_INT_PREFIX + const_CORE_SUFFIX)
            self.db_plugin_iface = dbus.Interface(self.db_object,
                                dbus_interface=const_INT_PREFIX + const_PLUGIN_SUFFIX)
        except dbus.exceptions.DBusException,e:
            if e._dbus_error_name=='org.freedesktop.DBus.Error.ServiceUnknown':
                raise BridgeExceptionNoService
            else:
                raise e
        #props = self.db_core_iface.getProperties()

    def register(self, functionName, handler, iface="core"):
        if iface == "core":
            self.db_core_iface.connect_to_signal(functionName, handler)
        elif iface == "plugin":
            self.db_plugin_iface.connect_to_signal(functionName, handler)
        else:
            error(_('Unknown interface'))

    def addContact(self, entity_jid, profile_key="@DEFAULT@"):
        return self.db_core_iface.addContact(entity_jid, profile_key)

    def asyncConnect(self, profile_key="@DEFAULT@", callback=None, errback=None):
        return self.db_core_iface.asyncConnect(profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:]))

    def asyncCreateProfile(self, profile, callback=None, errback=None):
        return self.db_core_iface.asyncCreateProfile(profile, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:]))

    def asyncGetParamA(self, name, category, attribute="value", profile_key="@DEFAULT@", callback=None, errback=None):
        return unicode(self.db_core_iface.asyncGetParamA(name, category, attribute, profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:])))

    def callMenu(self, category, name, menu_type, profile_key):
        return unicode(self.db_core_iface.callMenu(category, name, menu_type, profile_key))

    def confirmationAnswer(self, id, accepted, data):
        return self.db_core_iface.confirmationAnswer(id, accepted, data)

    def connect(self, profile_key="@DEFAULT@"):
        return self.db_core_iface.connect(profile_key)

    def createProfile(self, profile):
        return self.db_core_iface.createProfile(profile)

    def delContact(self, entity_jid, profile_key="@DEFAULT@"):
        return self.db_core_iface.delContact(entity_jid, profile_key)

    def deleteProfile(self, profile):
        return self.db_core_iface.deleteProfile(profile)

    def disconnect(self, profile_key="@DEFAULT@"):
        return self.db_core_iface.disconnect(profile_key)

    def getConfig(self, section, name):
        return unicode(self.db_core_iface.getConfig(section, name))

    def getContacts(self, profile_key="@DEFAULT@"):
        return self.db_core_iface.getContacts(profile_key)

    def getHistory(self, from_jid, to_jid, limit, between=True, callback=None, errback=None):
        return self.db_core_iface.getHistory(from_jid, to_jid, limit, between, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:]))

    def getLastResource(self, contact_jid, profile_key="@DEFAULT@"):
        return unicode(self.db_core_iface.getLastResource(contact_jid, profile_key))

    def getMenuHelp(self, category, name, menu_type):
        return unicode(self.db_core_iface.getMenuHelp(category, name, menu_type))

    def getMenus(self, ):
        return self.db_core_iface.getMenus()

    def getParamA(self, name, category, attribute="value", profile_key="@DEFAULT@"):
        return unicode(self.db_core_iface.getParamA(name, category, attribute, profile_key))

    def getParams(self, profile_key="@DEFAULT@", callback=None, errback=None):
        return unicode(self.db_core_iface.getParams(profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:])))

    def getParamsCategories(self, ):
        return self.db_core_iface.getParamsCategories()

    def getParamsForCategory(self, category, profile_key="@DEFAULT@", callback=None, errback=None):
        return unicode(self.db_core_iface.getParamsForCategory(category, profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:])))

    def getParamsUI(self, profile_key="@DEFAULT@", callback=None, errback=None):
        return unicode(self.db_core_iface.getParamsUI(profile_key, reply_handler=callback, error_handler=lambda err:errback(err._dbus_error_name[len(const_ERROR_PREFIX)+1:])))

    def getPresenceStatus(self, profile_key="@DEFAULT@"):
        return self.db_core_iface.getPresenceStatus(profile_key)

    def getProfileName(self, profile_key="@DEFAULT@"):
        return unicode(self.db_core_iface.getProfileName(profile_key))

    def getProfilesList(self, ):
        return self.db_core_iface.getProfilesList()

    def getProgress(self, id):
        return self.db_core_iface.getProgress(id)

    def getVersion(self, ):
        return unicode(self.db_core_iface.getVersion())

    def getWaitingSub(self, profile_key="@DEFAULT@"):
        return self.db_core_iface.getWaitingSub(profile_key)

    def isConnected(self, profile_key="@DEFAULT@"):
        return self.db_core_iface.isConnected(profile_key)

    def launchAction(self, action_type, data, profile_key="@DEFAULT@"):
        return unicode(self.db_core_iface.launchAction(action_type, data, profile_key))

    def registerNewAccount(self, login, password, email, host, port=5222):
        return unicode(self.db_core_iface.registerNewAccount(login, password, email, host, port))

    def sendMessage(self, to_jid, message, subject='', mess_type="chat", profile_key="@DEFAULT@"):
        return self.db_core_iface.sendMessage(to_jid, message, subject, mess_type, profile_key)

    def setParam(self, name, value, category, profile_key="@DEFAULT@"):
        return self.db_core_iface.setParam(name, value, category, profile_key)

    def setPresence(self, to_jid='', show='', priority=0, statuses={}, profile_key="@DEFAULT@"):
        return self.db_core_iface.setPresence(to_jid, show, priority, statuses, profile_key)

    def subscription(self, sub_type, entity, profile_key="@DEFAULT@"):
        return self.db_core_iface.subscription(sub_type, entity, profile_key)

    def updateContact(self, entity_jid, name, groups, profile_key="@DEFAULT@"):
        return self.db_core_iface.updateContact(entity_jid, name, groups, profile_key)


#methods from plugins
    def getRoomsJoined(self, profile_key):
        return self.db_plugin_iface.getRoomsJoined(profile_key)

    def getRoomsSubjects(self, profile_key):
        return self.db_plugin_iface.getRoomsSubjects(profile_key)

    def joinMUC(self, room_jid, nick, options, profile_key):
        if options == None:
            options = [('', '')] #XXX: we have to do this awful hack because python dbus need to guess the signature
        return self.db_plugin_iface.joinMUC(room_jid, nick, options, profile_key)

    def tarotGameLaunch(self, players, profile_key):
        return self.db_plugin_iface.tarotGameLaunch(players, profile_key)
    
    def tarotGameCreate(self, room_jid, players, profile_key):
        return self.db_plugin_iface.tarotGameCreate(room_jid, players, profile_key)

    def tarotGameReady(self, player, referee, profile_key):
        return self.db_plugin_iface.tarotGameReady(player, referee, profile_key)

    def tarotGameContratChoosed(self, player, referee, contrat, profile_key):
        return self.db_plugin_iface.tarotGameContratChoosed(player, referee, contrat, profile_key)

    def tarotGamePlayCards(self, player, referee, cards, profile_key):
        return self.db_plugin_iface.tarotGamePlayCards(player, referee, cards, profile_key)

    def quizGameLaunch(self, players, profile_key):
        return self.db_plugin_iface.quizGameLaunch(players, profile_key)
    
    def quizGameCreate(self, room_jid, players, profile_key):
        return self.db_plugin_iface.quizGameCreate(room_jid, players, profile_key)

    def quizGameReady(self, player, referee, profile_key):
        return self.db_plugin_iface.quizGameReady(player, referee, profile_key)

    def quizGameAnswer(self, player, referee, answer, profile_key):
        return self.db_plugin_iface.quizGameAnswer(player, referee, answer, profile_key)

    def radiocolLaunch(self, players, profile_key):
        return self.db_plugin_iface.radiocolLaunch(players, profile_key)
    
    def radiocolCreate(self, room_jid, profile_key):
        return self.db_plugin_iface.radiocolCreate(room_jid, profile_key)

    def radiocolSongAdded(self, room_jid, song_path, profile):
        return self.db_plugin_iface.radiocolSongAdded(room_jid, song_path, profile)

    def sendFile(self, to, path, data, profile_key):
        return self.db_plugin_iface.sendFile(to, path, data, profile_key)

    def pipeOut(self, to, path, data, profile_key):
        return self.db_plugin_iface.pipeOut(to, path, data, profile_key)

    def findGateways(self, target, profile_key):
        return self.db_plugin_iface.findGateways(target, profile_key)

    def getCard(self, target, profile_key):
        return self.db_plugin_iface.getCard(target, profile_key)

    def getCardCache(self, target, profile_key):
        return self.db_plugin_iface.getCardCache(target, profile_key)

    def getAvatarFile(self, hash):
        return self.db_plugin_iface.getAvatarFile(hash)

    def in_band_register(self, target, profile_key):
        return self.db_plugin_iface.in_band_register(target, profile_key)

    def gatewayRegister(self, action, target, data, profile_key):
        if data == None:
            data = [('', '')] #XXX: we have to do this awful hack because python dbus need to guess the signature
        return self.db_plugin_iface.gatewayRegister(action, target, data, profile_key)

    def getLastMicroblogs(self, jid, max_items, profile_key, callback=None, errback=None):
        return self.db_plugin_iface.getLastMicroblogs(jid, max_items, profile_key, reply_handler=callback, error_handler=errback)

    def sendGroupBlog(self, access_type, access_list, message, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.sendGroupBlog(access_type, access_list, message, profile_key)
    
    def getLastGroupBlogs(self, jid, max_items, profile_key, callback=None, errback=None):
        return self.db_plugin_iface.getLastGroupBlogs(jid, max_items, profile_key, reply_handler=callback, error_handler=errback)
    
    def getMassiveLastGroupBlogs(self, publishers_type, publishers, max_items=10, profile_key='@DEFAULT@', callback=None, errback=None):
        return self.db_plugin_iface.getMassiveLastGroupBlogs(publishers_type, publishers, max_items, profile_key, reply_handler=callback, error_handler=errback)
    
    def sendPersonalEvent(self, event_type, data, profile_key):
        return self.db_plugin_iface.sendPersonalEvent(event_type, data, profile_key)
    
    def setMicroblogAccess(self, access="presence", profile_key='@DEFAULT@', callback=None, errback=None):
        return self.db_plugin_iface.setMicroblogAccess(access, profile_key, reply_handler=callback, error_handler=errback)
